{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "# 翻译 PyTorch 代码"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 1,
   "metadata": {},
   "outputs": [],
   "source": [
    "import set_env"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "metadata": {},
   "outputs": [],
   "source": [
    "import numpy as np\n",
    "\n",
    "import torch\n",
    "from torch.nn import Module\n",
    "\n",
    "import tvm.testing\n",
    "from tvm.contrib.msc.framework.torch.frontend import translate\n",
    "from tvm.contrib.msc.framework.torch import codegen"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 10,
   "metadata": {},
   "outputs": [],
   "source": [
    "def verify_model(torch_model, input_info, via_relax=True):\n",
    "    \"\"\"Compare torch module results\"\"\"\n",
    "\n",
    "    graph, weights = translate.from_torch(torch_model, input_info, via_relax=via_relax)\n",
    "    model = codegen.to_torch(graph, weights)\n",
    "    # print(graph)\n",
    "    torch_datas = [torch.from_numpy(np.random.rand(*i[0]).astype(i[1])) for i in input_info]\n",
    "    with torch.no_grad():\n",
    "        golden = torch_model(*torch_datas)\n",
    "    with torch.no_grad():\n",
    "        if not graph.get_inputs():\n",
    "            result = model()\n",
    "        else:\n",
    "            result = model(*torch_datas)\n",
    "    if not isinstance(golden, (list, tuple)):\n",
    "        golden = [golden]\n",
    "    if not isinstance(result, (list, tuple)):\n",
    "        result = [result]\n",
    "    assert len(golden) == len(result), \"golden {} mismatch with result {}\".format(\n",
    "        len(golden), len(result)\n",
    "    )\n",
    "    for gol_r, new_r in zip(golden, result):\n",
    "        if isinstance(gol_r, torch.Tensor):\n",
    "            tvm.testing.assert_allclose(\n",
    "                gol_r.detach().numpy(), new_r.detach().numpy(), atol=1e-5, rtol=1e-5\n",
    "            )\n",
    "        else:\n",
    "            assert gol_r == new_r"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 4,
   "metadata": {},
   "outputs": [],
   "source": [
    "def test_conv1d():\n",
    "    \"\"\"test torch translator for conv1d\"\"\"\n",
    "\n",
    "    class Conv1D1(Module):\n",
    "        def __init__(self):\n",
    "            super().__init__()\n",
    "            self.conv = torch.nn.Conv1d(3, 6, 7, bias=True)\n",
    "\n",
    "        def forward(self, data):\n",
    "            return self.conv(data)\n",
    "\n",
    "    class Conv1D2(Module):\n",
    "        def __init__(self):\n",
    "            super().__init__()\n",
    "            self.conv = torch.nn.Conv1d(3, 6, 7, bias=False)\n",
    "\n",
    "        def forward(self, data):\n",
    "            return self.conv(data)\n",
    "\n",
    "    input_info = [([1, 3, 10], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Conv1D1(), input_info, via_relax)\n",
    "        verify_model(Conv1D2(), input_info, via_relax)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 5,
   "metadata": {},
   "outputs": [
    {
     "name": "stderr",
     "output_type": "stream",
     "text": [
      "/media/pc/data/lxw/ai/tvm/python/tvm/contrib/msc/framework/torch/codegen/codegen.py:74: FutureWarning: You are using `torch.load` with `weights_only=False` (the current default value), which uses the default pickle module implicitly. It is possible to construct malicious pickle data which will execute arbitrary code during unpickling (See https://github.com/pytorch/pytorch/blob/main/SECURITY.md#untrusted-models for more details). In a future release, the default value for `weights_only` will be flipped to `True`. This limits the functions that could be executed during unpickling. Arbitrary objects will no longer be allowed to be loaded via this mode unless they are explicitly allowlisted by the user via `torch.serialization.add_safe_globals`. We recommend you start setting `weights_only=True` for any use case where you don't have full control of the loaded file. Please open an issue on GitHub for any issues related to this experimental feature.\n",
      "  state_dict = torch.load(folder.relpath(graph.name + \".pth\"))\n",
      "/media/pc/data/lxw/ai/tvm/python/tvm/contrib/msc/framework/torch/codegen/codegen.py:74: FutureWarning: You are using `torch.load` with `weights_only=False` (the current default value), which uses the default pickle module implicitly. It is possible to construct malicious pickle data which will execute arbitrary code during unpickling (See https://github.com/pytorch/pytorch/blob/main/SECURITY.md#untrusted-models for more details). In a future release, the default value for `weights_only` will be flipped to `True`. This limits the functions that could be executed during unpickling. Arbitrary objects will no longer be allowed to be loaded via this mode unless they are explicitly allowlisted by the user via `torch.serialization.add_safe_globals`. We recommend you start setting `weights_only=True` for any use case where you don't have full control of the loaded file. Please open an issue on GitHub for any issues related to this experimental feature.\n",
      "  state_dict = torch.load(folder.relpath(graph.name + \".pth\"))\n"
     ]
    }
   ],
   "source": [
    "test_conv1d()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 6,
   "metadata": {},
   "outputs": [],
   "source": [
    "def test_conv2d():\n",
    "    \"\"\"test torch translator for conv2d\"\"\"\n",
    "\n",
    "    class Conv2D1(Module):\n",
    "        def __init__(self):\n",
    "            super().__init__()\n",
    "            self.conv = torch.nn.Conv2d(3, 6, 7, bias=True)\n",
    "\n",
    "        def forward(self, data):\n",
    "            return self.conv(data)\n",
    "\n",
    "    class Conv2D2(Module):\n",
    "        def __init__(self):\n",
    "            super().__init__()\n",
    "            self.conv = torch.nn.Conv2d(3, 6, 7, bias=False)\n",
    "\n",
    "        def forward(self, data):\n",
    "            return self.conv(data)\n",
    "\n",
    "    input_info = [([1, 3, 10, 10], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Conv2D1(), input_info, via_relax)\n",
    "        verify_model(Conv2D2(), input_info, via_relax)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 11,
   "metadata": {},
   "outputs": [],
   "source": [
    "test_conv2d()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "def test_linear():\n",
    "    \"\"\"test torch translator for linear\"\"\"\n",
    "\n",
    "    class Dense1(Module):\n",
    "        def __init__(self):\n",
    "            super().__init__()\n",
    "            self.linear = torch.nn.Linear(10, 7, bias=True)\n",
    "\n",
    "        def forward(self, data):\n",
    "            return self.linear(data)\n",
    "\n",
    "    class Dense2(Module):\n",
    "        def __init__(self):\n",
    "            super().__init__()\n",
    "            self.linear = torch.nn.Linear(10, 7, bias=False)\n",
    "\n",
    "        def forward(self, data):\n",
    "            return self.linear(data)\n",
    "\n",
    "    class MatMul1(Module):\n",
    "        def forward(self, x, y):\n",
    "            return torch.matmul(x, y)\n",
    "\n",
    "    input_info = [([1, 3, 10, 10], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Dense1(), input_info, via_relax)\n",
    "        verify_model(Dense2(), input_info, via_relax)\n",
    "        verify_model(MatMul1(), [([10, 10], \"float32\"), ([10, 10], \"float32\")], via_relax)\n",
    "\n",
    "\n",
    "def test_bmm():\n",
    "    \"\"\"test torch translator for bmm\"\"\"\n",
    "\n",
    "    class BMM(Module):\n",
    "        def forward(self, x, y):\n",
    "            return torch.bmm(x, y)\n",
    "\n",
    "    input_info = [((4, 128, 256), \"float32\"), ((4, 256, 512), \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(BMM(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_baddbmm():\n",
    "    \"\"\"test torch translator for baddbmm\"\"\"\n",
    "\n",
    "    class BAddBMM1(Module):\n",
    "        def forward(self, c, x, y):\n",
    "            return torch.baddbmm(c, x, y)\n",
    "\n",
    "    class BAddBMM2(Module):\n",
    "        def forward(self, c, x, y):\n",
    "            return torch.baddbmm(c, x, y, alpha=2, beta=0)\n",
    "\n",
    "    input_info = [\n",
    "        ((4, 128, 512), \"float32\"),\n",
    "        ((4, 128, 256), \"float32\"),\n",
    "        ((4, 256, 512), \"float32\"),\n",
    "    ]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(BAddBMM1(), input_info, via_relax)\n",
    "        verify_model(BAddBMM2(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_relu():\n",
    "    \"\"\"test torch translator for relu\"\"\"\n",
    "\n",
    "    class ReLU(Module):\n",
    "        def __init__(self):\n",
    "            super().__init__()\n",
    "            self.relu = torch.nn.ReLU()\n",
    "\n",
    "        def forward(self, data):\n",
    "            return self.relu(data)\n",
    "\n",
    "    class ReLU1(Module):\n",
    "        def forward(self, data):\n",
    "            return torch.nn.functional.relu(data)\n",
    "\n",
    "    input_info = [([10, 10], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(ReLU(), input_info, via_relax)\n",
    "        verify_model(ReLU1(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_relu6():\n",
    "    \"\"\"test torch translator for relu6\"\"\"\n",
    "\n",
    "    class ReLU6(Module):\n",
    "        def __init__(self):\n",
    "            super().__init__()\n",
    "            self.relu6 = torch.nn.ReLU6()\n",
    "\n",
    "        def forward(self, data):\n",
    "            return self.relu6(data)\n",
    "\n",
    "    input_info = [([10, 10], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(ReLU6(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_maxpool2d():\n",
    "    \"\"\"test torch translator for maxpool2d\"\"\"\n",
    "\n",
    "    class MaxPool2d(Module):\n",
    "        def __init__(self):\n",
    "            super().__init__()\n",
    "            self.pool = torch.nn.MaxPool2d(kernel_size=[1, 1])\n",
    "\n",
    "        def forward(self, data):\n",
    "            return self.pool(data)\n",
    "\n",
    "    class MaxPool2d2(Module):\n",
    "        def __init__(self):\n",
    "            super().__init__()\n",
    "            self.pool = torch.nn.MaxPool2d(kernel_size=[2, 2], dilation=[2, 3])\n",
    "\n",
    "        def forward(self, data):\n",
    "            return self.pool(data)\n",
    "\n",
    "    class MaxPool2d3(Module):\n",
    "        def __init__(self):\n",
    "            super().__init__()\n",
    "            self.pool = torch.nn.MaxPool2d(kernel_size=[4, 4], padding=2, stride=2)\n",
    "\n",
    "        def forward(self, data):\n",
    "            return self.pool(data)\n",
    "\n",
    "    input_info = [([1, 3, 10, 10], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(MaxPool2d(), input_info, via_relax)\n",
    "        verify_model(MaxPool2d2(), input_info, via_relax)\n",
    "        verify_model(MaxPool2d3(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_avgpool2d():\n",
    "    \"\"\"test torch translator for avgpool2d\"\"\"\n",
    "\n",
    "    class AvgPool2d(Module):\n",
    "        def __init__(self):\n",
    "            super().__init__()\n",
    "            self.pool = torch.nn.AvgPool2d(kernel_size=[1, 1])\n",
    "\n",
    "        def forward(self, data):\n",
    "            return self.pool(data)\n",
    "\n",
    "    class AvgPool2d2(Module):\n",
    "        def __init__(self):\n",
    "            super().__init__()\n",
    "            self.pool = torch.nn.AvgPool2d(kernel_size=[4, 4], stride=2, padding=2, ceil_mode=True)\n",
    "\n",
    "        def forward(self, data):\n",
    "            return self.pool(data)\n",
    "\n",
    "    input_info = [([1, 3, 10, 10], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(AvgPool2d(), input_info, via_relax)\n",
    "        verify_model(AvgPool2d2(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_adaptive_avgpool2d():\n",
    "    \"\"\"test torch translator for adaptive_avgpool2d\"\"\"\n",
    "\n",
    "    class AdaptiveAvgPool2d0(Module):\n",
    "        def __init__(self):\n",
    "            super().__init__()\n",
    "            self.pool = torch.nn.AdaptiveAvgPool2d([10, 10])\n",
    "\n",
    "        def forward(self, data):\n",
    "            return self.pool(data)\n",
    "\n",
    "    input_info = [([1, 3, 10, 10], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(AdaptiveAvgPool2d0(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_flatten():\n",
    "    \"\"\"test torch translator for flatten\"\"\"\n",
    "\n",
    "    class Flatten(Module):\n",
    "        def __init__(self):\n",
    "            super().__init__()\n",
    "            self.f = torch.nn.Flatten(2, -1)\n",
    "\n",
    "        def forward(self, data):\n",
    "            return self.f(data)\n",
    "\n",
    "    input_info = [([1, 3, 10, 10], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Flatten(), input_info, via_relax)\n",
    "        verify_model(torch.nn.Flatten(2, -1), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_batchnorm2d():\n",
    "    \"\"\"test torch translator for batchnorm2d\"\"\"\n",
    "\n",
    "    class BatchNorm2d(Module):\n",
    "        def __init__(self):\n",
    "            super().__init__()\n",
    "            self.batchnorm = torch.nn.BatchNorm2d(3)\n",
    "\n",
    "        def forward(self, data):\n",
    "            return self.batchnorm(data)\n",
    "\n",
    "    input_info = [([1, 3, 10, 10], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(BatchNorm2d(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_embedding():\n",
    "    \"\"\"test torch translator for embedding\"\"\"\n",
    "\n",
    "    class Embedding(Module):\n",
    "        def __init__(self):\n",
    "            super().__init__()\n",
    "            self.embedding = torch.nn.Embedding(10, 3)\n",
    "\n",
    "        def forward(self, data):\n",
    "            return self.embedding(data)\n",
    "\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Embedding(), [([4], \"int64\")], via_relax)\n",
    "        verify_model(Embedding(), [([4, 5], \"int64\")], via_relax)\n",
    "\n",
    "\n",
    "def test_layernorm():\n",
    "    \"\"\"test torch translator for layernorm\"\"\"\n",
    "\n",
    "    class LayerNorm(Module):\n",
    "        def __init__(self):\n",
    "            super().__init__()\n",
    "            self.layernorm = torch.nn.LayerNorm((10, 10))\n",
    "\n",
    "        def forward(self, data):\n",
    "            return self.layernorm(data)\n",
    "\n",
    "    input_info = [([1, 3, 10, 10], \"float32\")]\n",
    "    verify_model(LayerNorm(), input_info)\n",
    "\n",
    "\n",
    "def test_cross_entropy():\n",
    "    \"\"\"test torch translator for cross_entropy\"\"\"\n",
    "\n",
    "    class CrossEntropy1(Module):\n",
    "        def __init__(self):\n",
    "            super().__init__()\n",
    "            self.loss = torch.nn.CrossEntropyLoss()\n",
    "\n",
    "        def forward(self, logits, targets):\n",
    "            return self.loss(logits, targets)\n",
    "\n",
    "    class CrossEntropy2(Module):\n",
    "        def __init__(self):\n",
    "            super().__init__()\n",
    "            self.weight = torch.nn.Parameter(torch.ones((2,)))\n",
    "            self.loss = torch.nn.CrossEntropyLoss(weight=self.weight)\n",
    "\n",
    "        def forward(self, logits, targets):\n",
    "            return self.loss(logits, targets)\n",
    "\n",
    "    class CrossEntropy3(Module):\n",
    "        def __init__(self):\n",
    "            super().__init__()\n",
    "            self.loss = torch.nn.CrossEntropyLoss(ignore_index=1, reduction=\"sum\")\n",
    "\n",
    "        def forward(self, logits, targets):\n",
    "            return self.loss(logits, targets)\n",
    "\n",
    "    input_info = [([3, 2], \"float32\"), ([3], \"int64\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(CrossEntropy1(), input_info, via_relax)\n",
    "        verify_model(CrossEntropy2(), input_info, via_relax)\n",
    "        verify_model(CrossEntropy3(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_silu():\n",
    "    \"\"\"test torch translator for silu\"\"\"\n",
    "\n",
    "    class SiLU(Module):\n",
    "        def __init__(self):\n",
    "            super().__init__()\n",
    "            self.silu = torch.nn.SiLU()\n",
    "\n",
    "        def forward(self, data):\n",
    "            return self.silu(data)\n",
    "\n",
    "    class SiLU2(Module):\n",
    "        def forward(self, data):\n",
    "            return torch.nn.functional.silu(data)\n",
    "\n",
    "    input_info = [([1, 3, 10, 10], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(SiLU(), input_info, via_relax)\n",
    "        verify_model(SiLU2(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_groupnorm():\n",
    "    \"\"\"test torch translator for groupnorm\"\"\"\n",
    "\n",
    "    class GroupNorm(Module):\n",
    "        def __init__(self):\n",
    "            super().__init__()\n",
    "            self.groupnorm = torch.nn.GroupNorm(3, 3)\n",
    "\n",
    "        def forward(self, data):\n",
    "            return self.groupnorm(data)\n",
    "\n",
    "    input_info = [([1, 3, 10, 10], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(GroupNorm(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_softmax():\n",
    "    \"\"\"test torch translator for softmax\"\"\"\n",
    "\n",
    "    class Softmax(Module):\n",
    "        def __init__(self):\n",
    "            super().__init__()\n",
    "            self.softmax = torch.nn.Softmax(dim=1)\n",
    "\n",
    "        def forward(self, data):\n",
    "            return self.softmax(data)\n",
    "\n",
    "    input_info = [([1, 3, 10, 10], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Softmax(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_binary():\n",
    "    \"\"\"test torch translator for binary\"\"\"\n",
    "\n",
    "    input_info1 = [([1, 3, 10, 10], \"float32\"), ([1, 3, 10, 10], \"float32\")]\n",
    "    input_info2 = [([1, 3, 10, 10], \"float32\")]\n",
    "\n",
    "    # Add\n",
    "    class Add1(Module):\n",
    "        def forward(self, lhs, rhs):\n",
    "            return lhs + rhs\n",
    "\n",
    "    class Add2(Module):\n",
    "        def forward(self, lhs):\n",
    "            return lhs + 1.0\n",
    "\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Add1(), input_info1, via_relax)\n",
    "        verify_model(Add2(), input_info2, via_relax)\n",
    "\n",
    "    # Sub\n",
    "    class Sub1(Module):\n",
    "        def forward(self, lhs, rhs):\n",
    "            return lhs - rhs\n",
    "\n",
    "    class Sub2(Module):\n",
    "        def forward(self, lhs):\n",
    "            return lhs - 1.0\n",
    "\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Sub1(), input_info1, via_relax)\n",
    "        verify_model(Sub2(), input_info2, via_relax)\n",
    "\n",
    "    # Mul\n",
    "    class Mul1(Module):\n",
    "        def forward(self, lhs, rhs):\n",
    "            return lhs * rhs\n",
    "\n",
    "    class Mul2(Module):\n",
    "        def forward(self, lhs):\n",
    "            return lhs * 1.0\n",
    "\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Mul1(), input_info1, via_relax)\n",
    "        verify_model(Mul2(), input_info2, via_relax)\n",
    "\n",
    "    # True div\n",
    "    class TrueDiv1(Module):\n",
    "        def forward(self, lhs, rhs):\n",
    "            return lhs / rhs\n",
    "\n",
    "    class TrueDiv2(Module):\n",
    "        def forward(self, lhs):\n",
    "            return lhs / 1.0\n",
    "\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(TrueDiv1(), input_info1, via_relax)\n",
    "        verify_model(TrueDiv2(), input_info2, via_relax)\n",
    "\n",
    "    # Floor div\n",
    "    class FloorDiv1(Module):\n",
    "        def forward(self, lhs, rhs):\n",
    "            return lhs // rhs\n",
    "\n",
    "    class FloorDiv2(Module):\n",
    "        def forward(self, lhs):\n",
    "            return lhs // 1.0\n",
    "\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(FloorDiv1(), input_info1, via_relax)\n",
    "        verify_model(FloorDiv2(), input_info2, via_relax)\n",
    "\n",
    "    # Power\n",
    "    class Power1(Module):\n",
    "        def forward(self, lhs, rhs):\n",
    "            return lhs**rhs\n",
    "\n",
    "    class Power2(Module):\n",
    "        def forward(self, lhs):\n",
    "            return lhs**1.0\n",
    "\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Power1(), input_info1, via_relax)\n",
    "        verify_model(Power2(), input_info2, via_relax)\n",
    "\n",
    "    # LT\n",
    "    class LT1(Module):\n",
    "        def forward(self, lhs, rhs):\n",
    "            return lhs < rhs\n",
    "\n",
    "    class LT2(Module):\n",
    "        def forward(self, lhs):\n",
    "            return lhs < 1.0\n",
    "\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(LT1(), input_info1, via_relax)\n",
    "        verify_model(LT2(), input_info2, via_relax)\n",
    "\n",
    "\n",
    "def test_size():\n",
    "    \"\"\"test torch translator for size\"\"\"\n",
    "\n",
    "    class Size(Module):\n",
    "        def forward(self, data):\n",
    "            return data.size()\n",
    "\n",
    "    input_info = [([1, 3, 10, 10], \"float32\")]\n",
    "    verify_model(Size(), input_info)\n",
    "\n",
    "\n",
    "def test_squeeze():\n",
    "    \"\"\"test torch translator for squeeze\"\"\"\n",
    "\n",
    "    class Squeeze1(Module):\n",
    "        def forward(self, data):\n",
    "            return data.squeeze(1)\n",
    "\n",
    "    class Squeeze2(Module):\n",
    "        def forward(self, data):\n",
    "            return data.squeeze()\n",
    "\n",
    "    input_info = [([3, 1, 4, 1], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Squeeze1(), input_info, via_relax)\n",
    "        verify_model(Squeeze2(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_unsqueeze():\n",
    "    \"\"\"test torch translator for unsqueeze\"\"\"\n",
    "\n",
    "    class Unsqueeze1(Module):\n",
    "        def forward(self, data):\n",
    "            return data.unsqueeze(1)\n",
    "\n",
    "    class Unsqueeze2(Module):\n",
    "        def forward(self, data):\n",
    "            return data.unsqueeze(-1)\n",
    "\n",
    "    input_info = [([1, 3, 10, 10], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Unsqueeze1(), input_info, via_relax)\n",
    "        verify_model(Unsqueeze2(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_getattr():\n",
    "    \"\"\"test torch translator for getattr\"\"\"\n",
    "\n",
    "    class GetAttr1(Module):\n",
    "        def forward(self, data):\n",
    "            return data.shape\n",
    "\n",
    "    input_info = [([1, 3, 10, 10], \"float32\")]\n",
    "    verify_model(GetAttr1(), input_info)\n",
    "\n",
    "\n",
    "@pytest.mark.xfail(reason=\"MSC does not support Tuple of PrimValue\")\n",
    "def test_getitem():\n",
    "    \"\"\"test torch translator for getitem\"\"\"\n",
    "\n",
    "    # TODO(tong.meng): strided_slice reshape bug for x[0, 1::2, :, :3]\n",
    "    class Slice1(Module):\n",
    "        def forward(self, x):\n",
    "            return x[0:1, 1::2, :, :3]\n",
    "\n",
    "    class Slice2(Module):\n",
    "        def forward(self, x):\n",
    "            return x[:, None, None, :, None]\n",
    "\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Slice1(), [([1, 3, 10, 10], \"float32\")], via_relax)\n",
    "        verify_model(Slice2(), [([8, 16], \"float32\")], via_relax)\n",
    "\n",
    "\n",
    "def test_unary():\n",
    "    \"\"\"test torch translator for unary\"\"\"\n",
    "\n",
    "    input_info = [([1, 3, 10, 10], \"float32\")]\n",
    "\n",
    "    # sin\n",
    "    class Sin(Module):\n",
    "        def forward(self, data):\n",
    "            return torch.sin(data)\n",
    "\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Sin(), input_info, via_relax)\n",
    "\n",
    "    # cos\n",
    "    class Cos(Module):\n",
    "        def forward(self, data):\n",
    "            return torch.cos(data)\n",
    "\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Cos(), input_info, via_relax)\n",
    "\n",
    "    # exp\n",
    "    class Exp(Module):\n",
    "        def forward(self, data):\n",
    "            return torch.exp(data)\n",
    "\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Exp(), input_info, via_relax)\n",
    "\n",
    "    # sqrt\n",
    "    class Sqrt(Module):\n",
    "        def forward(self, data):\n",
    "            return torch.sqrt(data)\n",
    "\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Sqrt(), input_info, via_relax)\n",
    "\n",
    "    # sigmoid\n",
    "    class Sigmoid(Module):\n",
    "        def forward(self, data):\n",
    "            return torch.sigmoid(data)\n",
    "\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Sigmoid(), input_info, via_relax)\n",
    "\n",
    "    # round\n",
    "    class Round(Module):\n",
    "        def forward(self, data):\n",
    "            return torch.round(data)\n",
    "\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Round(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_gelu():\n",
    "    \"\"\"test torch translator for gelu\"\"\"\n",
    "\n",
    "    class Gelu(Module):\n",
    "        def forward(self, data):\n",
    "            return torch.nn.functional.gelu(data)\n",
    "\n",
    "    input_info = [([1, 3, 10, 10], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Gelu(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_tanh():\n",
    "    \"\"\"test torch translator for tanh\"\"\"\n",
    "\n",
    "    class Tanh(Module):\n",
    "        def forward(self, data):\n",
    "            return torch.tanh(data)\n",
    "\n",
    "    input_info = [([1, 3, 10, 10], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Tanh(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_clamp():\n",
    "    \"\"\"test torch translator for clamp\"\"\"\n",
    "\n",
    "    class Clamp(Module):\n",
    "        def forward(self, data):\n",
    "            return torch.clamp(data, min=0.1, max=0.5)\n",
    "\n",
    "    input_info = [([1, 3, 10, 10], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Clamp(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_interpolate():\n",
    "    \"\"\"test torch translator for interpolate\"\"\"\n",
    "\n",
    "    class Interpolate(Module):\n",
    "        def forward(self, data):\n",
    "            return torch.nn.functional.interpolate(data, (5, 5))\n",
    "\n",
    "    input_info = [([1, 3, 10, 10], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Interpolate(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_addmm():\n",
    "    \"\"\"test torch translator for addmm\"\"\"\n",
    "\n",
    "    class Addmm(Module):\n",
    "        def forward(self, x_1, x_2, x_3):\n",
    "            return torch.addmm(x_1, x_2, x_3)\n",
    "\n",
    "    input_info = [\n",
    "        ([10, 10], \"float32\"),\n",
    "        ([10, 10], \"float32\"),\n",
    "        ([10, 10], \"float32\"),\n",
    "    ]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Addmm(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_split():\n",
    "    \"\"\"test torch translator for split\"\"\"\n",
    "\n",
    "    class Split(Module):\n",
    "        def forward(self, data):\n",
    "            return torch.split(data, 1, dim=1)\n",
    "\n",
    "    input_info = [([1, 3, 10, 10], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Split(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_cumsum():\n",
    "    \"\"\"test torch translator for cumsum\"\"\"\n",
    "\n",
    "    class Cumsum(Module):\n",
    "        def forward(self, data):\n",
    "            return torch.cumsum(data, dim=1, dtype=torch.int32)\n",
    "\n",
    "    input_info = [([1, 2, 3, 4], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Cumsum(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_chunk():\n",
    "    \"\"\"test torch translator for chunk\"\"\"\n",
    "\n",
    "    class Chunk(Module):\n",
    "        def forward(self, data):\n",
    "            return torch.chunk(data, 3, dim=1)\n",
    "\n",
    "    input_info = [([1, 3, 10, 10], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Chunk(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_inplace_fill():\n",
    "    \"\"\"test torch translator for inplace_fill\"\"\"\n",
    "\n",
    "    class InplaceFill(Module):\n",
    "        def forward(self, data):\n",
    "            data.fill_(1.5)\n",
    "            return data\n",
    "\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(InplaceFill(), [([10, 10], \"float32\")], via_relax)\n",
    "\n",
    "\n",
    "def test_arange():\n",
    "    \"\"\"test torch translator for arange\"\"\"\n",
    "\n",
    "    # pylint: disable=unused-argument\n",
    "    class Arange(Module):\n",
    "        def forward(self, data):\n",
    "            return torch.arange(0, 20, dtype=torch.int32)\n",
    "\n",
    "    verify_model(Arange(), [([10, 10], \"float32\")])\n",
    "\n",
    "\n",
    "def test_tril():\n",
    "    \"\"\"test torch translator for tril\"\"\"\n",
    "\n",
    "    class Tril(Module):\n",
    "        def forward(self, data):\n",
    "            return torch.tril(data, 1)\n",
    "\n",
    "    class InplaceTril(Module):\n",
    "        def forward(self, data):\n",
    "            data.tril_(1)\n",
    "            return data\n",
    "\n",
    "    input_info = [([10, 10], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Tril(), input_info, via_relax)\n",
    "        verify_model(InplaceTril(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_triu():\n",
    "    \"\"\"test torch translator for triu\"\"\"\n",
    "\n",
    "    class Triu(Module):\n",
    "        def forward(self, data):\n",
    "            return torch.triu(data, 1)\n",
    "\n",
    "    class InplaceTriu(Module):\n",
    "        def forward(self, data):\n",
    "            data.triu_(1)\n",
    "            return data\n",
    "\n",
    "    input_info = [([10, 10], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Triu(), input_info, via_relax)\n",
    "        verify_model(InplaceTriu(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_new_ones():\n",
    "    \"\"\"test torch translator for new_ones\"\"\"\n",
    "\n",
    "    class NewOnes(Module):\n",
    "        def forward(self, x):\n",
    "            return x.new_ones(1, 2, 3)\n",
    "\n",
    "    input_info = [([1, 2, 3], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(NewOnes(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_expand():\n",
    "    \"\"\"test torch translator for expand\"\"\"\n",
    "\n",
    "    class Expand(Module):\n",
    "        def forward(self, x):\n",
    "            return x.expand(4, 2, 3, 4)\n",
    "\n",
    "    input_info = [([1, 2, 3, 4], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Expand(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_reduce():\n",
    "    \"\"\"test torch translator for reduce\"\"\"\n",
    "\n",
    "    # sum\n",
    "    class Sum(Module):\n",
    "        def forward(self, x):\n",
    "            return torch.sum(x, (2, 1))\n",
    "\n",
    "    # max\n",
    "    class Max(Module):\n",
    "        def forward(self, x):\n",
    "            return torch.max(x)\n",
    "\n",
    "    # min\n",
    "    class Min(Module):\n",
    "        def forward(self, x):\n",
    "            return torch.min(x)\n",
    "\n",
    "    input_info = [([1, 2, 3, 4], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Sum(), input_info, via_relax)\n",
    "    verify_model(Max(), input_info, False)\n",
    "    verify_model(Min(), input_info, False)\n",
    "\n",
    "\n",
    "def test_datatype():\n",
    "    \"\"\"test torch translator for datatype\"\"\"\n",
    "\n",
    "    input_info = [([1, 2, 3, 4], \"float32\")]\n",
    "\n",
    "    # float\n",
    "    class ToFloat(Module):\n",
    "        def forward(self, x):\n",
    "            return x.float()\n",
    "\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(ToFloat(), input_info, via_relax)\n",
    "\n",
    "    # half\n",
    "    class ToHalf(Module):\n",
    "        def forward(self, x):\n",
    "            return x.half()\n",
    "\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(ToHalf(), input_info, via_relax)\n",
    "\n",
    "    # type\n",
    "    class Type(Module):\n",
    "        def forward(self, x):\n",
    "            return x.type(torch.float32)\n",
    "\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Type(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_permute():\n",
    "    \"\"\"test torch translator for permute\"\"\"\n",
    "\n",
    "    class Permute(Module):\n",
    "        def forward(self, x):\n",
    "            return x.permute(0, 3, 2, 1)\n",
    "\n",
    "    input_info = [([1, 2, 3, 4], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Permute(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_reshape():\n",
    "    \"\"\"test torch translator for reshape\"\"\"\n",
    "\n",
    "    class Reshape(Module):\n",
    "        def forward(self, x):\n",
    "            return x.reshape(2, 12)\n",
    "\n",
    "    input_info = [([1, 2, 3, 4], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Reshape(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_transpose():\n",
    "    \"\"\"test torch translator for transpose\"\"\"\n",
    "\n",
    "    class Transpose(Module):\n",
    "        def forward(self, x):\n",
    "            return x.transpose(1, 3)\n",
    "\n",
    "    input_info = [([1, 2, 3, 4], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Transpose(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_view():\n",
    "    \"\"\"test torch translator for view\"\"\"\n",
    "\n",
    "    class View(Module):\n",
    "        def forward(self, x):\n",
    "            return x.view(2, 12)\n",
    "\n",
    "    input_info = [([1, 2, 3, 4], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(View(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_keep_params():\n",
    "    \"\"\"test torch translator for keep_params\"\"\"\n",
    "\n",
    "    class Conv2D1(Module):\n",
    "        def __init__(self):\n",
    "            super().__init__()\n",
    "            self.conv = torch.nn.Conv2d(3, 6, 7, bias=True)\n",
    "\n",
    "        def forward(self, data):\n",
    "            return self.conv(data)\n",
    "\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Conv2D1(), [([1, 3, 10, 10], \"float32\")], via_relax)\n",
    "\n",
    "\n",
    "def test_unwrap_unit_return_tuple():\n",
    "    \"\"\"test torch translator for unwrap_unit_return_tuple\"\"\"\n",
    "\n",
    "    class Identity(Module):\n",
    "        def forward(self, x):\n",
    "            return (x,)\n",
    "\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Identity(), [([256, 256], \"float32\")], via_relax)\n",
    "\n",
    "\n",
    "def test_no_bind_return_tuple():\n",
    "    \"\"\"test torch translator for no_bind_return_tuple\"\"\"\n",
    "\n",
    "    class Identity(Module):\n",
    "        def forward(self, x, y):\n",
    "            return (x, y)\n",
    "\n",
    "    input_info = [([256, 256], \"float32\"), ([256, 256], \"float32\")]\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Identity(), input_info, via_relax)\n",
    "\n",
    "\n",
    "def test_argmax():\n",
    "    \"\"\"test torch translator for argmax\"\"\"\n",
    "\n",
    "    class Argmax1(Module):\n",
    "        def forward(self, data):\n",
    "            return torch.argmax(data, dim=-1)\n",
    "\n",
    "    class Argmax2(Module):\n",
    "        def forward(self, data):\n",
    "            return torch.argmax(data, dim=-1, keepdim=True)\n",
    "\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Argmax1(), [([256, 256], \"float32\")], via_relax)\n",
    "        verify_model(Argmax2(), [([256, 256], \"float32\")], via_relax)\n",
    "\n",
    "\n",
    "def test_argmin():\n",
    "    \"\"\"test torch translator for argmin\"\"\"\n",
    "\n",
    "    class Argmin1(Module):\n",
    "        def forward(self, data):\n",
    "            return torch.argmin(data)\n",
    "\n",
    "    class Argmin2(Module):\n",
    "        def forward(self, data):\n",
    "            return torch.argmin(data, keepdim=True)\n",
    "\n",
    "    verify_model(Argmin1(), [([256, 256], \"float32\")])\n",
    "    verify_model(Argmin2(), [([256, 256], \"float32\")])\n",
    "\n",
    "\n",
    "def test_to():\n",
    "    \"\"\"test torch translator for to\"\"\"\n",
    "\n",
    "    class To1(Module):\n",
    "        def forward(self, data):\n",
    "            return data.to(torch.float16)\n",
    "\n",
    "    class To2(Module):\n",
    "        def forward(self, data):\n",
    "            return data.to(\"cpu\")\n",
    "\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(To1(), [([256, 256], \"float32\")], via_relax)\n",
    "        verify_model(To2(), [([256, 256], \"float32\")], via_relax)\n",
    "\n",
    "\n",
    "def test_mean():\n",
    "    \"\"\"test torch translator for mean\"\"\"\n",
    "\n",
    "    class Mean(Module):\n",
    "        def forward(self, data):\n",
    "            return data.mean(-1)\n",
    "\n",
    "    class MeanKeepDim(Module):\n",
    "        def forward(self, data):\n",
    "            return data.mean(-1, keepdim=True)\n",
    "\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Mean(), [([256, 256], \"float32\")], via_relax)\n",
    "        verify_model(MeanKeepDim(), [([256, 256], \"float32\")], via_relax)\n",
    "\n",
    "\n",
    "def test_rsqrt():\n",
    "    \"\"\"test torch translator for rsqrt\"\"\"\n",
    "\n",
    "    class Rsqrt(Module):\n",
    "        def forward(self, data):\n",
    "            return torch.rsqrt(data)\n",
    "\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Rsqrt(), [([256, 256], \"float32\")], via_relax)\n",
    "\n",
    "\n",
    "def test_neg():\n",
    "    \"\"\"test torch translator for neg\"\"\"\n",
    "\n",
    "    class Neg(Module):\n",
    "        def forward(self, data):\n",
    "            return -data\n",
    "\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Neg(), [([256, 256], \"float32\")], via_relax)\n",
    "\n",
    "\n",
    "def test_max():\n",
    "    \"\"\"test torch translator for max\"\"\"\n",
    "\n",
    "    class Max(Module):\n",
    "        def forward(self, x, y):\n",
    "            return torch.max(x, y)\n",
    "\n",
    "    for via_relax in [True, False]:\n",
    "        verify_model(Max(), [([256, 256], \"float32\"), ([256, 256], \"float32\")], via_relax)\n",
    "\n",
    "\n",
    "def test_attention():\n",
    "    \"\"\"test torch translator for attention\"\"\"\n",
    "\n",
    "    # pylint: disable=import-outside-toplevel\n",
    "    import torch.nn.functional as F\n",
    "\n",
    "    class Attention1(Module):\n",
    "        def forward(self, q_data, k_data, v_data):\n",
    "            return F.scaled_dot_product_attention(q_data, k_data, v_data)\n",
    "\n",
    "    class Attention2(Module):\n",
    "        def forward(self, q_data, k_data, v_data):\n",
    "            return F.scaled_dot_product_attention(q_data, k_data, v_data, is_causal=True)\n",
    "\n",
    "    input_info = [\n",
    "        ([32, 8, 128, 64], \"float32\"),\n",
    "        ([32, 8, 128, 64], \"float32\"),\n",
    "        ([32, 8, 128, 64], \"float32\"),\n",
    "    ]\n",
    "    verify_model(Attention1(), input_info)\n",
    "    verify_model(Attention2(), input_info)\n",
    "\n",
    "    class Attention3(Module):\n",
    "        def forward(self, q_data, k_data, v_data, mask):\n",
    "            return F.scaled_dot_product_attention(q_data, k_data, v_data, mask)\n",
    "\n",
    "    verify_model(\n",
    "        Attention3(),\n",
    "        [\n",
    "            ([32, 8, 128, 64], \"float32\"),\n",
    "            ([32, 8, 128, 64], \"float32\"),\n",
    "            ([32, 8, 128, 64], \"float32\"),\n",
    "            ([32, 8, 128, 128], \"float32\"),\n",
    "        ],\n",
    "    )\n",
    "\n",
    "\n",
    "if __name__ == \"__main__\":\n",
    "    tvm.testing.main()\n"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "xin",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.12.4"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 2
}
